# -*- coding: utf-8 -*-

import os


TESTS = os.path.dirname(os.path.realpath(__file__))
ROOT = os.path.realpath(os.path.join(TESTS, '..'))
RESOURCES = os.path.join(TESTS, 'resources')
SANDBOX = os.path.join(TESTS, 'sandbox')

class config:
    URL = None
    PORT = None




# -*- coding: utf-8 -*-
#
# pytest plugins
#
# ------------------------------------------------


# imports
# -------
import pytest
from . import config


# config
# ------
def pytest_addoption(parser):
    return


def pytest_configure(config):
    return


# fixtures
# --------
@pytest.fixture(autouse=True, scope='session')
def bootstrap():
    """
    Global test setup/teardown fixture.
    """
    from faux.utils import free_port
    config.PORT = free_port()
    config.URL = 'http://localhost:{}'.format(config.PORT)

    yield

    import os
    import shutil
    from . import SANDBOX
    if os.path.exists(SANDBOX):
        shutil.rmtree(SANDBOX)
    return


@pytest.fixture(scope='session')
def server():
    """
    Set up mock server for testing request caching.
    """

    # imports
    import logging
    from faux import Server
    from flask import request
    from . import config, RESOURCES
    
    # set up app
    app = Server(__name__, cache=RESOURCES)
    app.logger.setLevel(logging.INFO)


    # define routes for testing
    @app.route('/simple', methods=['GET', 'POST', 'PUT', 'DELETE'])
    def simple():
        """
        Simple endpint with get/post
        """
        ret = {
            'status': 'ok',
            'uuid': '{{uuid}}',
        }
        if request.args:
            # account for flask version changing
            # request.args structure
            up = {
                k: v[0] if isinstance(v, (list, tuple)) else v
                for k, v in request.args.items()
            }
            ret.update(up)
        if request.json:
            ret.update(request.json)
        return ret

    @app.route('/nested/<param>', methods=['GET', 'POST', 'PUT', 'DELETE'])
    def nested(param):
        """
        Manage server state.
        """
        ret = {
            'status': 'ok',
            'param': param,
            'uuid': '{{uuid}}'
        }
        if request.args:
            # account for flask version changing
            # request.args structure
            up = {
                k: v[0] if isinstance(v, (list, tuple)) else v
                for k, v in request.args.items()
            }
            ret.update(up)
        if request.json:
            ret.update(request.json)
        return ret


    # run server in the background for testing
    with app.run(port=config.PORT, debug=True):
        yield
    return



# -*- coding: utf-8 -*-
#
# Testing for server caching.
# 
# ------------------------------------------------


# imports
# -------
import unittest
import pytest
import requests
from xml.etree import ElementTree
from . import config


# tests
# -----
@pytest.mark.usefixtures("server")
class TestLocalMock(unittest.TestCase):

    def test_no_file(self):
        response = requests.get(config.URL + '/missing')
        self.assertEqual(response.status_code, 404)
        return

    def test_status(self):
        response = requests.get(config.URL + '/_/status')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        return
    
    def test_get_root(self):
        # json
        response = requests.get(config.URL + '/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['type'], 'json')

        # xml
        response = requests.get(config.URL + '/xml')
        self.assertEqual(response.status_code, 200)
        data = ElementTree.fromstring(response.content)
        self.assertEqual(data.find('status').text, 'ok')
        self.assertEqual(data.find('type').text, 'xml')

        # text
        response = requests.get(config.URL + '/txt')
        self.assertEqual(response.status_code, 200)
        data = str(response.text.rstrip())
        self.assertEqual(data, 'test')
        return

    def test_get_query(self):
        # json
        response = requests.get(config.URL + '/query/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['type'], 'json')

        # xml
        response = requests.get(config.URL + '/query/xml')
        self.assertEqual(response.status_code, 200)
        data = ElementTree.fromstring(response.content)
        self.assertEqual(data.find('status').text, 'ok')
        self.assertEqual(data.find('type').text, 'xml')

        # text
        response = requests.get(config.URL + '/query/txt')
        self.assertEqual(response.status_code, 200)
        data = str(response.text.rstrip())
        self.assertEqual(data, 'test')
        return

    def test_get_params_single(self):
        response = requests.get(config.URL + '?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    def test_get_params_multiple(self):
        response = requests.get(config.URL + '?narg=ntest&arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    def test_get_query_params_single(self):
        response = requests.get(config.URL + '/query?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    def test_get_query_params_multiple(self):
        response = requests.get(config.URL + '/query?narg=ntest&arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    def test_post_payload(self):
        # test resource: 91cc355
        response = requests.post(config.URL, json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        return

    def test_post_params_payload(self):
        # test resource: arg=test91cc355
        response = requests.post(config.URL + '?arg=test', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return
 
    def test_post_query_payload(self):
        # test resource: query/91cc355
        response = requests.post(config.URL + '/query', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        return

    def test_post_query_params_payload(self):
        # test resource: query/arg=test91cc355
        response = requests.post(config.URL + '/query?arg=test', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    def test_get_named_param(self):
        import uuid
        uu = str(uuid.uuid4())

        # root
        response = requests.get(config.URL + '/' + uu)
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertTrue('uuid' in data)

        # query
        response = requests.get(config.URL + '/query/' + uu)
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertTrue('uuid' in data)
        return

    def test_resolve_with_post_method(self):
        # GET
        response = requests.get(config.URL + '/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['type'], 'json')

        # POST
        response = requests.post(config.URL + '/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['type'], 'json')

        # PUT
        response = requests.put(config.URL + '/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['type'], 'json')
        return



@pytest.mark.usefixtures("server")
class TestDynamicMock(unittest.TestCase):

    # GET
    def test_get_simple(self):
        response = requests.get(config.URL + '/simple')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        return 

    def test_get_simple_args(self):
        response = requests.get(config.URL + '/simple?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['arg'], 'test')
        return

    def test_get_nested(self):
        response = requests.get(config.URL + '/nested/test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        return

    def test_get_nested_args(self):
        response = requests.get(config.URL + '/nested/test?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['arg'], 'test')
        return

    # POST
    def test_post_simple(self):
        response = requests.post(config.URL + '/simple', json={'param': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        return

    def test_post_simple_args(self):
        response = requests.post(config.URL + '/simple?arg=test', json={'param': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['arg'], 'test')
        self.assertEqual(data['param'], 'test')
        return

    def test_post_nested(self):
        response = requests.post(config.URL + '/nested/test', json={'payload': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['payload'], 'test')
        return

    def test_post_nested_args(self):
        response = requests.post(config.URL + '/nested/test?arg=test', json={'payload': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['arg'], 'test')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['payload'], 'test')
        return

    # PUT
    def test_put_simple(self):
        response = requests.put(config.URL + '/simple', json={'param': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        return

    def test_put_simple_args(self):
        response = requests.put(config.URL + '/simple?arg=test', json={'param': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['arg'], 'test')
        self.assertEqual(data['param'], 'test')
        return

    def test_put_nested(self):
        response = requests.put(config.URL + '/nested/test', json={'payload': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['payload'], 'test')
        return

    def test_put_nested_args(self):
        response = requests.put(config.URL + '/nested/test?arg=test', json={'payload': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data['status'], 'ok')
        self.assertEqual(data['arg'], 'test')
        self.assertEqual(data['param'], 'test')
        self.assertEqual(data['payload'], 'test')
        return



# -*- coding: utf-8 -*-
#
# Testing for server caching.
# 
# ------------------------------------------------


# imports
# -------
import os
import json
import unittest
import pytest
from xml.etree import ElementTree

import faux
from faux import requests

from . import config, SANDBOX


# config
# ------
faux.cache(SANDBOX)


# tests
# -----
@pytest.mark.usefixtures("server")
class TestStatic(unittest.TestCase):

    def test_get_root(self):
        # json
        response = requests.get(config.URL + '/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'json'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['type'], local['type'])

        # xml
        response = requests.get(config.URL + '/xml')
        self.assertEqual(response.status_code, 200)
        data = ElementTree.fromstring(response.content)
        local = ElementTree.parse(os.path.join(SANDBOX, 'GET', 'xml')).getroot()
        self.assertEqual(data.find('status').text, local.find('status').text)
        self.assertEqual(data.find('type').text, local.find('type').text)

        # text
        response = requests.get(config.URL + '/txt')
        self.assertEqual(response.status_code, 200)
        data = str(response.text.rstrip())
        with open(os.path.join(SANDBOX, 'GET', 'txt'), 'r' ) as fi:
            local = fi.read()
        self.assertEqual(data, local)
        return

    def test_get_query(self):
        # json
        response = requests.get(config.URL + '/query/json')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'query', 'json'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['type'], local['type'])

        # xml
        response = requests.get(config.URL + '/query/xml')
        self.assertEqual(response.status_code, 200)
        data = ElementTree.fromstring(response.content)
        local = ElementTree.parse(os.path.join(SANDBOX, 'GET', 'query', 'xml')).getroot()
        self.assertEqual(data.find('status').text, local.find('status').text)
        self.assertEqual(data.find('type').text, local.find('type').text)

        # text
        response = requests.get(config.URL + '/query/txt')
        self.assertEqual(response.status_code, 200)
        data = str(response.text.rstrip())
        with open(os.path.join(SANDBOX, 'GET', 'query', 'txt'), 'r' ) as fi:
            local = fi.read()
        self.assertEqual(data, local)
        return

    def test_get_params_single(self):
        response = requests.get(config.URL + '?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'arg=test'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return

    def test_get_params_multiple(self):
        response = requests.get(config.URL + '?narg=ntest&arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'arg=test&narg=ntest'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return

    def test_get_query_params_single(self):
        response = requests.get(config.URL + '/query?arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'query', 'arg=test'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return

    def test_get_query_params_multiple(self):
        response = requests.get(config.URL + '/query?narg=ntest&arg=test')
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'GET', 'query', 'arg=test&narg=ntest'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return

    def test_post_payload(self):
        # test resource: 91cc355
        response = requests.post(config.URL, json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'POST', '91cc355'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        return

    def test_post_params_payload(self):
        # test resource: arg=test91cc355
        response = requests.post(config.URL + '?arg=test', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'POST', 'arg=test91cc355'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return
 
    def test_post_query_payload(self):
        # test resource: query/91cc355
        response = requests.post(config.URL + '/query', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'POST', 'query', '91cc355'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        return

    def test_post_query_params_payload(self):
        # test resource: query/arg=test91cc355
        response = requests.post(config.URL + '/query?arg=test', json={'data': 'test'})
        self.assertEqual(response.status_code, 200)
        data = response.json()
        with open(os.path.join(SANDBOX, 'POST', 'query', 'arg=test91cc355'), 'r' ) as fi:
            local = json.load(fi)
        self.assertEqual(data['status'], local['status'])
        self.assertEqual(data['param'], local['param'])
        self.assertEqual(data['arg'], local['arg'])
        return




# -*- coding: utf-8 -*-
#
# Main entry point
#
# ------------------------------------------------


# imports
# -------
import os
import sys
import argparse
import logging

from . import __version__, Server



# args
# ----
parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers()


# version
# -------
parser_version = subparsers.add_parser('version')
parser_version.set_defaults(func=lambda x: sys.stdout.write(__version__ + '\n'))


# status
# ------
def status(args):
    """
    Check if faux server is up and running.
    """
    import requests

    # set up connection params
    url = 'https' if args.ssl else 'http'
    url += '://' + args.host
    if args.port:
        url += ':' + str(args.port)

    # send and parse response
    response = requests.get(url + '/_/status')
    if response.status_code == 200:
        data = response.json()
        if data['status'] != 'ok':
            sys.stderr.write('Could not connect! See `faux serve` for instructions on how to run an instance.\n\n')
            sys.exit(1)
    sys.stdout.write(str(data) + '\n')
    return


parser_status = subparsers.add_parser('status')
parser_status.add_argument('-S', '--ssl', action='store_true', help='Use ssl for connecting to server.', default=False)
parser_status.add_argument('-H', '--host', type=str, help='Host to check.', default='127.0.0.1')
parser_status.add_argument('-P', '--port', type=int, help='Port to check.', default=9001)
parser_status.set_defaults(func=status)


# serve
# ------
def serve(args):
    """
    Run faux server with specified directory structure.
    """
    import time

    # assertions
    if not os.path.exists(args.path):
        raise AssertionError("Error: specified directory to serve does not exist!")

    # set log level
    logging.basicConfig(level=getattr(logging, args.log_level))

    # run server and wait for timeout
    app = Server(args.name, cache=args.path)
    with app.run(port=args.port):
        count = 0
        while True:
            time.sleep(1)
            count += 1
            if args.timeout is not None and count > args.timeout:
                break
    return


parser_serve = subparsers.add_parser('serve')
parser_serve.add_argument('-P', '--port', type=int, help='Port to run server on.', default=9001)
parser_serve.add_argument('-n', '--name', type=str, help='Optional name for server.', default=__name__)
parser_serve.add_argument('-t', '--timeout', type=int, help='Timeout for stopping server (seconds).', default=None)
parser_serve.add_argument('-l', '--log-level', help='Logging verbosity (DEBUG, INFO, ERROR, WARNING, CRITICAL, etc ...). Default is INFO', default='INFO')
parser_serve.add_argument('path', help='Directory structure to serve.')
parser_serve.set_defaults(func=serve)


# exec
# ----
def main():
    # parsing
    args = parser.parse_args()
    if not hasattr(args, 'func'):
        parser.print_help()
        sys.exit(0)

    # go!
    args.func(args)
    return


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Testing for server caching.
# 
# ------------------------------------------------


# imports
# -------
import unittest
import pytest
from faux.utils import format_data
import json


# tests
# -----
class TestFormatData(unittest.TestCase):

    def test_json(self):
        data = {
            'uuid': 'a{{uuid}}asdf',
            'name': '{{name }}aasdf',
            'text': 'basfg{{ text}}',
            'address': 'asdf{{ address }}asdf',
            'none': 'name'
        }
        res = json.loads(format_data(json.dumps(data)), strict=False)
        self.assertNotEqual(res['uuid'], data['uuid'])
        self.assertNotEqual(res['name'], data['name'])
        self.assertNotEqual(res['text'], data['name'])
        self.assertNotEqual(res['address'], data['address'])
        self.assertEqual(res['none'], data['none'])
        return

    def test_xml(self):
        # TODO: THIS
        return




# -*- coding: utf-8 -*-
#
# Utility methods for module.
#
# ----------------------------------


# imports
# -------
import re
import hashlib
from faker import Faker


# config
# ------
factory = Faker()
factory.uuid = factory.uuid4


# functions
# ---------
def format_data(string):
    for outer, inner in re.findall(r'({{.*?(\w+).*?}})', string):
        if hasattr(factory, inner):
            inner = getattr(factory, inner)()
            string = string.replace(outer, inner)
    return string


def free_port():
    import socket
    s = socket.socket()
    s.bind(('', 0))
    return s.getsockname()[1]


def request2path(url, args=None, payload=None):
    """
    Encode request into filepath recognizable by
    framework when serving requests.
    """
    path = url + '/'

    # convert args into queryable filename
    if args:
        for key in sorted(args.keys()):
            path += key + '=' + args[key] + '&'
        path = path[:-1]

    # convert payload into queryable hash postfix
    if payload:
        res = re.sub(r"\s+", "", ''.join(sorted(str(payload))))
        hc = hashlib.md5(res.encode('ascii')).hexdigest()
        hc = hc[:7]
        path += hc
    
    # final formatting
    if path[-1] == '/':
        path = path[:-1]
    return path


#!/usr/bin/env python
#
# Simple REST server.
#
# @author <bprinty@gmail.com>
# ---------------------------------------------------


# imports
# -------
import os
import requests
from functools import wraps
try:
    from urllib.parse import urlsplit, urlunsplit, parse_qsl
except ImportError:
    from urlparse import urlsplit, urlunsplit, parse_qsl

from .utils import request2path


# config
# ------
CACHE = None


def cache(path):
    """
    Method for setting options during request session.

    Example:
        >>> from faux import requests
        >>> requests.cache('/path/to/local/cache')
        >>> requests.get('http://localhost')
    """
    global CACHE
    if not path:
        raise AssertionError('Invalid path specified for cache directory ("{}")!'.format(path))
    CACHE = path
    return


# decorators
# ----------
def cache_response(func):
    """
    Decorator for caching requests submitted with this module
    into a file structure that can be served by a mock server.
    """

    @wraps(getattr(requests, func.__name__))
    def _(*args, **kwargs):
        global CACHE
        response = func(*args, **kwargs)
        if CACHE:

            # generate local url
            url = urlsplit(args[0])
            rpath = request2path(
                url=url.path,
                args=dict(parse_qsl(url.query)),
                payload=response.request.body
            )
            # for some reason, os.path.join(cache, method, rpath) doesn't work?
            path = os.path.join(CACHE, response.request.method.upper()) + '/' + rpath
            
            # make location if it doesn't exist
            directory, filename = os.path.dirname(path), os.path.basename(path)
            if not os.path.exists(directory):
                os.makedirs(directory)

            # write the data
            with open(path, 'w') as fo:
                fo.write(response.text)

        return response
    return _


# functions
# ---------
@cache_response
def get(*args, **kwargs):
    return requests.get(*args, **kwargs)


@cache_response
def post(*args, **kwargs):
    return requests.post(*args, **kwargs)


@cache_response
def put(*args, **kwargs):
    return requests.put(*args, **kwargs)


@cache_response
def delete(*args, **kwargs):
    return requests.delete(*args, **kwargs)




# -*- coding: utf-8 -*-
#
# testing for entry points
#
# ------------------------------------------------


# imports
# -------
import os
import unittest
import pytest
import subprocess
import time
import json
import ast
import requests

from faux import __version__
from faux import __main__ as cli
from faux.utils import free_port

from . import RESOURCES, SANDBOX, ROOT


# tests
# -----
class TestEntryPoints(unittest.TestCase):

    def call(self, subcommand, *args):
        return subprocess.check_output('PYTHONPATH=$PYTHONPATH:{} python -m faux {} {}'.format(
            ROOT, subcommand, ' '.join(args)
        ), stderr=subprocess.STDOUT, shell=True)

    def popen(self, subcommand, *args):
        return subprocess.Popen('PYTHONPATH=$PYTHONPATH:{} python -m faux {} {}'.format(
            ROOT, subcommand, ' '.join(args)
        ), stderr=subprocess.STDOUT, stdout=subprocess.PIPE, shell=True)

    def test_version(self):
        res = self.call('version').decode('utf-8')
        self.assertEqual(__version__, res.rstrip())
        return

    def test_serve_status(self):
        # start server with 3 second timeout
        port = str(free_port())
        self.popen('serve', '-t', '3', '-P', port, RESOURCES)
        time.sleep(1)
        
        # query status
        res = self.call('status', '-P', port).decode('utf-8')
        data = ast.literal_eval(res) # not sure why json.loads doesn't work here.
        self.assertEqual(data['status'], 'ok')

        # test with request
        result = requests.get('http://127.0.0.1:{}/json'.format(port))
        self.assertEqual(result.status_code, 200)
        self.assertEqual(result.json()['status'], 'ok')
        return



# -*- coding: utf-8 -*-
#
# Classes for setting up server mock.
#
# -----------------------------------


# imports
# -------
import os
import re
import six
import json
import time
import logging
import hashlib
from functools import wraps
from xml.etree import ElementTree
from threading import Thread


from .utils import format_data, request2path


# classes
# -------
class Server(object):
    """
    Object mimicking flask server to allow for spinning
    up server mock.
    """

    def __init__(self, *args, **kwargs):
        from flask import Flask

        self.cache = kwargs.pop('cache', None)
        self.flask = Flask(*args, **kwargs)
        # self.flask.logger.setLevel(logging.ERROR)
        if self.cache:
            self.init()
        return

    def init(self):
        """
        Method for decorating custom url handlers on server.
        """
        from flask import request, jsonify

        # general-purpose request handler
        @self.route('/<path:path>', methods=['GET', 'POST', 'PUT', 'DELETE'])
        def general(path):
            # format path
            path = request2path(path, args=request.args, payload=request.data)
            filename = os.path.join(self.cache, path)

            # if file doesn't exist, try to prepend the request method
            if not os.path.exists(filename):
                filename = os.path.join(self.cache, request.method.upper(), path)

            # if file exists, read and return
            if os.path.exists(filename) and not os.path.isdir(filename):
                with open(filename, 'r') as fi:
                    data = fi.read()
                return data

            return jsonify({'error': 'Could not find local resource!'}), 404


        # requests on root url
        @self.route('/', methods=['GET', 'POST', 'PUT', 'DELETE'])
        def root():
            return general('.')


        # requests on root url with uuid
        @self.route('/<uuid:uuid>', methods=['GET', 'POST', 'PUT', 'DELETE'])
        def ident(uuid):
            return general('_uuid')


        # requests on nested path with uuid
        @self.route('/<path:path>/<uuid:uuid>', methods=['GET', 'POST', 'PUT', 'DELETE'])
        def path_ident(path, uuid):
            return general(path + '/_uuid')


        # status url
        @self.route('/_/status', methods=['GET'])
        def status():
            return {"status": "ok"}

        return

    @property
    def logger(self):
        """
        Expose flask logger so user can change settings.

        TODO: update this class to use __getattr__ for defaulting
              to internal getattr(self.flask, item)
        """
        return self.flask.logger

    def route(self, *args, **kwargs):
        """
        Override flask route decorator to provide easier
        UX for return data. With these changes, users can
        simply return a dictionary or xml Element object instead
        of needing to craft a full response.
        """
        from flask import jsonify, Response

        # inner decorator to make return values easier
        def inner(method):
            @wraps(method)
            def _(*args, **kwargs):
                ret = method(*args, **kwargs)
                
                # json
                if isinstance(ret, dict):
                    ret = format_data(json.dumps(ret))
                    ret = json.loads(ret, strict=False)
                    return jsonify(ret), 200
                
                # xml
                elif isinstance(ret, ElementTree.Element):
                    ret = ElementTree.tostring(ret, encoding='utf8', method='xml')
                    ret = format_data(ret)
                    return Response(ret, mimetype='text/xml')
                
                # string
                elif isinstance(ret, six.string_types):
                    ret = format_data(ret)
                    return ret, 200

                # other data
                else:
                    return ret
            return _

        # proxy for using normal flask route decorator
        def decorator(func):
            return self.flask.route(*args, **kwargs)(inner(func))

        return decorator

    def run(self, **kwargs):
        # TODO: decorate all routes for reading from cached directory
        return Instance(self.flask, **kwargs)

    def __exit__(self, type, value, traceback):
        return


class Instance(object):
    """
    Contextmanager for running managing server mock.
    """

    def __init__(self, app, **kwargs):
        self.app = app
        kwargs['use_reloader'] = False
        self.thread = Thread(target=self.app.run, kwargs=kwargs)
        self.thread.daemon = True
        return

    def __enter__(self):
        self.thread.start()
        time.sleep(1)
        return self.thread

    def __exit__(self, type, value, traceback):
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# package setup
# 
# @author <bprinty@gmail.com>
# ------------------------------------------------


# config
# ------
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import faux


# files
# -----
with open('requirements.txt') as fi:
    requirements = fi.readlines()

with open('tests/requirements.txt') as fi:
    test_requirements = fi.readlines()

with open('README.rst') as readme_file:
    readme = readme_file.read()


# exec
# ----
setup(
    name='faux',
    version=faux.__version__,
    description="Quick and easy server mocking.",
    long_description=readme,
    author="bprinty",
    author_email='bprinty@gmail.com',
    url='https://github.com/bprinty/faux',
    packages=[
        'faux',
    ],
    package_dir={'faux':
                 'faux'},
    include_package_data=True,
    install_requires=requirements,
    keywords='faux',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='nosetests',
    tests_require=test_requirements,
    entry_points={
        'console_scripts': [
            'faux = faux.__main__:main'
        ],
    }
)



# -*- coding: utf-8 -*-

__author__ = 'bprinty'
__email__ = 'bprinty@gmail.com'
__version__ = '0.0.4'


from .server import Server
from . import client as requests
from .client import get, post, put, delete, cache
